/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Annozilla.
 *
 * The Initial Developer of the Original Code is
 * Matthew Wilson <matthew@mjwilson.demon.co.uk>.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   C.Greg Hagerty
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function closeWindow(){
    setTimeout("window.close()",500);
}

/**
 * @param annotation if updating, an annotation.  if new post, then null.
 * @param baseURL
 * @param xptr if updating, then null.  if new post, then xptr string.
 * @param bodyURI
 * @param inReplyTo
 * @param inReplyToBodyURI
 * @param threadRoot
 * @param target
 */
function CommonEditor (annotation, baseURL, xptr, bodyURI, inReplyTo, inReplyToBodyURI, threadRoot, target) {
    this.prefs = new AnnozillaPrefs(this);
    this.bundle = srGetStrBundle(
        "chrome://annozilla/locale/annozilla.properties"
    );
    this.editorBundle = srGetStrBundle(
        "chrome://annozilla/locale/editor.properties"
    );

    document.getElementById("annotationType").database.
        AddDataSource(window.opener.gAnnozilla.annotationTypes);
    
    this.inReplyTo = inReplyTo;
    this.inReplyToBodyURI = inReplyToBodyURI;
    this.threadRoot = threadRoot;
    
    var defaultCreator  = this.prefs.getPref("creator");
    var defaultLanguage = this.prefs.getPref("language");
    var defaultCharset;
    
    // Any Dublin Core title present in the annotation
    var dcTitle = undefined;

    // annotation may also be type of new annotation (PostObserver takes care)
    if (annotation && (typeof annotation != 'string')) {
        // defaultCharset will not be used by PostObserver.

        defaultCharset = null;
        this.updating = true;
        
        dcTitle = annotation.data.getLiteralProperty (
            PostObserver.DC10_NS, "title");

    } else {
        // defaultCharset will be used by PostObserver.

        defaultCharset = window.QueryInterface(
            Components.interfaces.nsIDOMWindow
        ).QueryInterface(
            Components.interfaces.nsIInterfaceRequestor
        ).getInterface(
            Components.interfaces.nsIDocCharset
        ).charset;
        this.updating = false;
    }
    
    // This produces an update function, which will be called at
    // startup, and after an operation succeeds or fails
    this.makeUpdate = function() {
        var t = this;
        return function(message, operation, success) {

            document.getElementById("posting").mode = "determined";
            document.getElementById("cancel").disabled = false;
            document.getElementById("statusbar-text").setAttribute(
                "label", message);

            // If the operation failed, nothing more to do
            if (operation != PostObserver.NONE && success == false)
            {
                return;
            }

            // For a successfully saved annotation, we can update it
            if (operation == PostObserver.POST || operation == PostObserver.REPLY || operation == PostObserver.UPDATE) {
                t.updating = true;

                // TODO we don't need two buttons anymore
                document.getElementById("post").disabled = true;
                document.getElementById("post").setAttribute("style", "display: none");
                document.getElementById("update").disabled = false;
                document.getElementById("update").setAttribute("style", "display: block");
            }

            if (operation != PostObserver.DELETE)
            {
                document.getElementById("annotationLanguage").value = t.observer.language;
                document.getElementById("annotationAuthor").value = t.observer.creator;

                // Need to set the annotation type after a pause, possibly
                // because we have to wait for the datasource to be built from
                // the RDF
                window.setTimeout
                    ( function ()
                      { if (t.observer.fullType != undefined) {
                              document.getElementById('annotationType').value = t.observer.fullType;
                          }
                      },
                      100);
            }

            // If we just saved or updated an annotation, we can delete it
            if (operation == PostObserver.POST || operation == PostObserver.UPDATE || operation == PostObserver.REPLY)
            {
                document.getElementById("delete").setAttribute("style", "display: block");
            }
            // If we just finished a delete, save, delete, and reply are no longer available
            else if (success && operation == PostObserver.DELETE)
            {
                document.getElementById("delete").disabled = true;
                document.getElementById("reply").disabled = true;
                document.getElementById("post").disabled = true;
                document.getElementById("update").disabled = true;
            }
        };
    };

    this.setElementsToDefaultValues = function(){
        var doc = document;

        // updating
        if (this.updating) {
            doc.getElementById("post").setAttribute  ("style", "display: none");
            doc.getElementById("update").setAttribute("style", "display: block");
            doc.getElementById("delete").setAttribute("style", "display: block");
            doc.getElementById("reply").setAttribute ("style", "display: block");
            this.setupServers(true, this.observer.annotation.serverName);

            if (this.threadRoot == undefined)
            {
                doc.getElementById("annotationType").setAttribute (
                    "ref",
                    "chrome://annozilla/content/resources/annotationTypes.rdf#annotationTypes");
            }
            else
            {
                doc.getElementById("annotationType").setAttribute (
                    "ref",
                    "chrome://annozilla/content/resources/annotationTypes.rdf#replyTypes");
            }
        }
        // posting or replying
        else {
            doc.getElementById("post").setAttribute("style",  "display: block");
            doc.getElementById("update").setAttribute("style","display: none");
            doc.getElementById("delete").setAttribute("style","display: none");
            doc.getElementById("reply").setAttribute("style","display: none");
            if (inReplyTo != undefined)
            {
                // replying
                this.setupServers(true, this.observer.server);
                doc.getElementById("annotationType").setAttribute (
                    "ref",
                    "chrome://annozilla/content/resources/annotationTypes.rdf#replyTypes");
            }
            else
            {
                // posting
                this.setupServers(false, this.observer.server);
                doc.getElementById("annotationType").setAttribute (
                    "ref",
                    "chrome://annozilla/content/resources/annotationTypes.rdf#annotationTypes");
            }
        }

        // This is an encoding of an empty HTML document with title "Annotation"
        var emptyPage = 
           'data:text/html;charset=utf-8;base64,PGh0bWw%2BDQogPGhlYWQ%2BDQogIDx0aXRsZT5Bbm5vdGF0aW9uPC90aXRsZT4NCiA8L2hlYWQ%2BDQogPGJvZHk%2BDQogPC9ib2R5Pg0KPC9odG1sPg0K';
        if (inReplyTo != undefined)
        {
            // This is an encoding of an empty HTML document with title "Annotation reply"
            emptyPage =
               'data:text/html;charset=utf-8;base64,PGh0bWw+DQogPGhlYWQ+DQogIDx0aXRsZT5Bbm5vdGF0aW9uIHJlcGx5PC90aXRsZT4NCiA8L2hlYWQ+DQogPGJvZHk+DQogPC9ib2R5Pg0KPC9odG1sPg0K';
        }

        // setup the HTML editor
        var editURI = this.observer.bodyURI ? this.observer.bodyURI : emptyPage;

        var self=this;
 
        doc.documentElement.setAttribute ("class", "loading");
        this.editor.loadUrl(editURI, function () {
                // Called after annotation body has finished loading
                var title = dcTitle ? dcTitle : self.editor.getTitle();
                document.getElementById ("annotationTitle").value = title;
                document.title = title;
                doc.documentElement.setAttribute ("class", "loaded");
            }
        );
    };

    this.observer = new PostObserver(  // This calls this.makeUpdate() to generate an update function.
        annotation,
        baseURL,
        xptr,
        bodyURI,
        this.makeUpdate(),
        defaultCreator,
        defaultLanguage,
        defaultCharset,
        inReplyTo,
        inReplyToBodyURI,
        threadRoot,
        target
    );

    this.observer.update("", PostObserver.NO_OPERATION);

    this.editor = getEditorWrapper();

    this.setElementsToDefaultValues();

    // set a window closing observer
    var that = this;
    window.addEventListener(
        "unload",
        function() { that.windowClosing(); },
        false
    );
}

CommonEditor.prototype = {};

CommonEditor.prototype.windowClosing = function() {
    this.observer.stopObserving();
    this.prefs.destructor();
};

/**
 * @param onlyOneServerAllowed boolean whether only the defaultServer may be used
 * @param defaultServer string name of defaultServer to select, or null if none
 */
CommonEditor.prototype.setupServers = function (
    onlyOneServerAllowed,
    defaultServer){

    // get the preferences directory
    var fileLocator = Components.classes[
        "@mozilla.org/file/directory_service;1"
    ].getService(Components.interfaces.nsIProperties);  
    var file = fileLocator.get("ProfD", Components.interfaces.nsIFile);
    
    // append servers filename
    file.append(PostObserver.SERVERS_FILENAME);	
    
    // get the data source
    var urlSpec = CommonEditor.getURLSpecFromFile(file);
    var ds = gRDF.GetDataSourceBlocking(urlSpec);
    
    // intialize the listbox
    var list = document.getElementById("serverList");
    list.database.AddDataSource(ds);
    list.setAttribute("ref", PostObserver.SERVERS_ROOT_URN);

    if (defaultServer) {
	// this is the bad thing that happens when 
	// interface designers don't put length fields
	// on lists (bad mozilla, bad)
	for(var i=0; true; i++) {
	    list.selectedIndex = i;
	    if (list.value == defaultServer) {
		break;
	    }
	}
    }
    else {
	list.selectedIndex = 0;
    }

    if (onlyOneServerAllowed) {
	list.disabled = true;
    }
};
   

CommonEditor.prototype.editorOnLoad = function(url, listener) {
  var editorShell = document.getElementById("edit-frame").editorShell;
  editorShell.editorType = 'html';
  editorShell.webShellWindow = window.frames[0];
  editorShell.contentWindow  = window.frames[0];
  editorShell.RegisterDocumentStateListener(listener);
  editorShell.contentsMIMEType = "text/html";
  editorShell.LoadUrl(url);
  
  return editorShell;
};

CommonEditor.prototype.cancel = function() {
    if (onclose()) {
        closeWindow();
    }
};

CommonEditor.prototype.deleteAnnotation = function() {
    
    var doc = document;
    // animate progress bar
    doc.getElementById("posting").mode = "undetermined";
    doc.getElementById("statusbar-text").setAttribute(
            "label", this.editorBundle.GetStringFromName("DELETING_STATUS"));
    
    this.observer.deleteAnnotation();

};

CommonEditor.prototype.post = function() {
    
    gDirty = false;
    
    var doc = document;
    // animate progress bar
    doc.getElementById("posting").mode = "undetermined";
    
    // disable the post and cancel buttons
    doc.getElementById("post").disabled = true;
    doc.getElementById("update").disabled = true;
    doc.getElementById("cancel").disabled = true;

    doc.getElementById("statusbar-text").setAttribute(
        "label",
        this.editorBundle.GetStringFromName("POSTING_STATUS")
    );

    var xml = this.editor.getDocument();

    this.observer.server = doc.getElementById("serverList").value;
    this.observer.language = doc.getElementById("annotationLanguage").value;
    this.observer.type = doc.getElementById("annotationType").value;
    this.observer.title = doc.getElementById ("annotationTitle").value;
//    this.observer.charset = defaultCharset;

    var result;
    try{
        result = this.observer.post(xml);
    }
    catch(e){
        throw "Exception while posting the annotation: " + e;
    }
    if(!result){
        window.alert(
            this.editorBundle.formatStringFromName(
                (this.observer.annotation)?("SERVER_ERROR1"):("SERVER_ERROR2"),
                [this.observer.annotation.serverURI],
                1
            )
        );
    }
};

CommonEditor.prototype.setTitle = function (title)
{
    document.title = title;
    this.editor.setTitle (title);
}

/*
CommonEditor.prototype.reply = function() {
    
    Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService)
      .logStringMessage("CommonEditor.reply");

    var doc = document;
    // animate progress bar
    doc.getElementById("posting").mode = "undetermined";
    
    // disable the post and cancel buttons
    doc.getElementById("reply").disabled = true;
    doc.getElementById("post").disabled = true;
    doc.getElementById("update").disabled = true;
    doc.getElementById("cancel").disabled = true;

    doc.getElementById("statusbar-text").setAttribute(
        "label",
        "Replying" // TODO i18n
    );

    var xml = this.editor.getDocument();

    this.observer.server = doc.getElementById("serverList").value;
    this.observer.language = doc.getElementById("annotationLanguage").value;
    this.observer.type = doc.getElementById("annotationType").value;
//    this.observer.charset = defaultCharset;

    var result;
    try{
        result = this.observer.reply(xml);
    }
    catch(e){
        throw "Exception while posting the annotation: " + e;
    }
    if(!result){
        window.alert(
            this.editorBundle.formatStringFromName(
                (this.observer.annotation)?("SERVER_ERROR1"):("SERVER_ERROR2"),
                [this.observer.annotation.serverURI],
                1
            )
        );
    }
};
*/

CommonEditor.prototype.hideItem = function(id)
{
  var item = document.getElementById(id);
  if (item) {
    item.setAttribute("hidden", "true");
  }
};

// from editorUtilities.js
CommonEditor.prototype.setElementEnabled = function (element, doEnable)
{
  if ( element ) {
    if ( doEnable ) {
      element.removeAttribute("disabled");
    }
    else {
      element.setAttribute("disabled", "true");
    }
  }
  else
  {
    dump("Element not found in SetElementEnabled\n");
  }
};


CommonEditor.getURLSpecFromFile = function(file) {
    
    if (Components.classes["@mozilla.org/network/io-service;1"] && Components.interfaces.nsIIOService) {
	
	var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
	var fileHandler = ioService.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);

	// new version
	if (fileHandler.getURLSpecFromFile) {
	    return fileHandler.getURLSpecFromFile(file);
	}
	// old version in ioService
	else {
	    return ioService.getURLSpecFromFile(file);
	}
    }
    // really old version
    else {
	var file_url = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIFileURL);
	file_url.file = file;
	return file_url.spec;
    }
};
