/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Annozilla.
 *
 * The Initial Developer of the Original Code is
 * Matthew Wilson <matthew@mjwilson.demon.co.uk>.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var gEditor;
var gInReplyTo;
var gInReplyToBody;
var gTarget;
var gThreadRoot;
var gDirty = false;

function editorOnload() {
    try{
        if(window.arguments && (window.arguments.length >= 8)) {
            gEditor = new CommonEditor(
                window.arguments[0], // annotation
                window.arguments[1], // baseURL
                window.arguments[2], // xptr
                window.arguments[3], // bodyURI
                window.arguments[4], // inReplyTo
                window.arguments[5], // inReplyToBodyURI
                window.arguments[6], // threadRoot
                window.arguments[7]  // target
            );
            
            if (window.arguments[0] != undefined)
            {
                gInReplyTo = window.arguments[0].annotationURI;
                gInReplyToBody = window.arguments[3];
            }
            gThreadRoot = window.arguments[6];
            gTarget = window.arguments[7];

        } else {
            var editorBundle = srGetStrBundle(
                "chrome://annozilla/locale/editor.properties"
            );
            window.alert (editorBundle.GetStringFromName ("ARGS_ERROR"));
            window.close();
        }
    } catch (e) {
        window.alert (e);
    }
}

function changed (e) {
    gDirty = true;
}

function onclose (e) {
    if (gDirty)
    {
        var editorBundle = srGetStrBundle("chrome://annozilla/locale/editor.properties");

        var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                    .getService(Components.interfaces.nsIPromptService);

        var flags=promptService.BUTTON_TITLE_YES * promptService.BUTTON_POS_0 +
                  promptService.BUTTON_TITLE_NO * promptService.BUTTON_POS_1;

        var choice = promptService.confirmEx
            (window,
             editorBundle.GetStringFromName("CONFIRM_CLOSE_TITLE"),
             editorBundle.GetStringFromName("CONFIRM_CLOSE_MESSAGE"),
             flags, null, null, null, null, {});

        if (choice == 0)
        {
          return true;
        }
        else
        {
            if (e != undefined) {
               e.preventDefault();
            }
            return false;
        }
    } else {
        return true;
    }
}

function designModeOn () {
    try {
        document.getElementById ("edit-frame").contentWindow.document.designMode = "on";
        
        document.getElementById ("edit-frame").contentWindow.document.addEventListener
            ("DOMSubtreeModified", function(event) { changed (event) }, false);
            // TODO remove event listener
        window.addEventListener ("close", function(event) { onclose (event) }, false);
        setup();

        document.getElementById ("boldButton").disabled = false;
        document.getElementById ("italicButton").disabled = false;
        document.getElementById ("underlineButton").disabled = false;
        document.getElementById ("ulButton").disabled = false;
        document.getElementById ("olButton").disabled = false;
        document.getElementById ("post").disabled = false;
        document.getElementById ("update").disabled = false;
        document.getElementById ("reply").disabled = false;
    } catch (e) {
        Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService).
      logStringMessage (e);
        var editorBundle = srGetStrBundle("chrome://annozilla/locale/editor.properties");
        window.alert (editorBundle.GetStringFromName ("SETUP_ERROR"));
    }
}

// Carry out an editor command
function simpleCommand (command) {
    document.getElementById("edit-frame").contentWindow.
        document.execCommand(command, false, null);
}

function post(){
    return gEditor.post();
}

/* function reply(){
    return gEditor.reply();
} */

function launchReplyWindow (inReplyToURI, inReplyToBodyURI, target)
{
    // If the annotation has a thread root set, use it.
    // Otherwise, we are editing an annotation, not a reply,
    // so use this annotation's URI as the root if we create any replies.
    var threadRoot = gThreadRoot ? gThreadRoot : inReplyToURI;

    window.opener.top.gAnnozilla.openEditorWindow
        ("chrome,resizable,dialog=no",
         null, // annotation
         inReplyToURI, // baseURL
         "xpointer(/html/body)", // TODO fragment?
         null, // bodyURI
         inReplyToURI, // inReplyTo
         inReplyToBodyURI,
         threadRoot,
         target);
}
