/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Annozilla.
 *
 * The Initial Developer of the Original Code is
 * Matthew Wilson <matthew@mjwilson.demon.co.uk>.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Dr. Katsuhiko Tsujino
 *   C. Greg Hagerty
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const ANNOZILLA_XUL_NS = "http://annozilla.mozdev.org/xul/2005";
const SEAMONKEY_ID = "{92650c4d-4b8e-4d2a-b7eb-24ecf4f6b63a}";

var gAnnozillaSidebar;

var browserWindowNamespace = window.parent;

var Annozilla = browserWindowNamespace.Annozilla;

function initializeAnnozillaSidebar () {

    if(!browserWindowNamespace.gAnnozilla) {
        browserWindowNamespace.initialize();
    }

    if (! Annozilla._hasRequiredComponents()) {
        document.getElementById ("loadCreateBox").setAttribute ("style", "display: none");
        document.getElementById ("noerrors").setAttribute ("style", "display: none");
        document.getElementById ("error").removeAttribute ("style");
        return;
    }

    browserWindowNamespace.gAnnozilla.setDefaultBodyDestination(
        document.getElementById("body")
    );
    
    gAnnozillaSidebar = new AnnozillaSidebar(browserWindowNamespace.gAnnozilla);
}

function createElement (tagName) {
    return document.createElementNS("http://www.w3.org/1999/xhtml", tagName);
}

// Called to inform the sidebar of the user's language preference
function setLanguage (language) {
}

// Internal helper function to find the UI element for a given annotation
// uri is the URI identifying the annotation
// first=true to find the first element identifying the annotation
//   (which contains the text of the annotated section of the document)
// or first=false to find the second such element
//   (with the text "[Annotation type] created by [author]...")
function findAnnotation (uri, first) {
    var foundFirst = false;
    var root = document.getElementById("annotationTreeChildren");
    var treeitems = root.getElementsByTagName("treeitem");
    for (var ix=0; ix < treeitems.length; ix++) {
        if (treeitems[ix].getAttributeNS (ANNOZILLA_XUL_NS, "annozilla-resource") == uri) {
            // If we want the first item, or if we have already found the
            // first item, then this is the one we want
            if (first || foundFirst) {
                return treeitems[ix]; // TODO why did this use to have .parentNode;
            // Otherwise...
            } else {
                foundFirst = true;
            }
        }
    }
    return undefined;
}

// For uncertain reasons "findAnnotation" used to find the
// node and then return its parent node. Keep the old implementation
// in here.
function findAnnotationParent (uri, first)
{
    var found = findAnnotation (uri, first);
    return (found == undefined) ? undefined: found.parentNode;
}

// Clear the annotations list from the sidebar.
// annotationURI: undefined to clear the whole list,
// or a URI to specify the URI of an annotation to clear the replies from
function clearList (annotationURI) {
    var children;
    if (annotationURI == undefined) {
        children = document.getElementById("annotationTreeChildren");
        // If we are clearing the entire list then also remove the
        // context and double-click options while there is nothing there
        // (because we will add the "No annotations" message while loading)
        try {
            children.removeAttribute ("context");
            children.removeAttribute ("ondblclick");
        } catch (e) {
        }
        while (children.hasChildNodes()) {
            children.removeChild (children.firstChild);
        }
    } else {
        var children = findAnnotationParent (annotationURI, false);
        while (children.childNodes.length > 1) {
            children.removeChild (children.lastChild);
        }
    }
}

function appendMessageToList (message) {
    var children = document.getElementById("annotationTreeChildren");

    var item = document.createElement ("treeitem");
    var row  = document.createElement ("treerow");
    var cell = document.createElement ("treecell");
    cell.setAttribute ("label", message);

    row.appendChild (cell);
    item.appendChild (row);
    children.appendChild (item);
}

function setHeader (message) {
    var header = document.getElementById("headerXUL");
    header.setAttribute ("label",message);
}

// Called when the user changes the auto-load preference
function autoLoadPrefChanged (autoload) {
}

// Called when the user changes the "load in new window" preference (obsolete)
function loadInNewWindowPrefChanged (loadInNewWindow) {
}

function addNewAnnotation (annotation, range, inReplyTo) {

    // The root of the annotation tree
    var root = document.getElementById("annotationTreeChildren");
 
    // The node to which we will attach the annotation
    var parent = root;
    if (inReplyTo != undefined) {
        // look for parent: the second treeitem element in document order
        // which has annozilla-resource equal to inReplyTo
        parent = findAnnotationParent (inReplyTo, false);

        if (parent == null)
        {
            Components.classes['@mozilla.org/consoleservice;1'].
                getService(Components.interfaces.nsIConsoleService).
                logStringMessage ("sidebar addNewAnnotation error: couldn't find parent for " + inReplyTo);
            return;
        }
    }

    var item = document.createElement ("treeitem");
    item.setAttribute ("container", "true");
    //CGH: prefers them closed
    //item.setAttribute ("open", "true");
    item.setAttribute ("open", "false");
    item.setAttributeNS
          (ANNOZILLA_XUL_NS,
           "annozilla-resource",
           annotation.annotationURI);
    item.setAttributeNS
          (ANNOZILLA_XUL_NS,
           "annozilla-body",
           annotation.bodyURI);
    item.setAttributeNS
          (ANNOZILLA_XUL_NS,
           "annozilla-in-reply-to",
           inReplyTo);
    var row  = document.createElement ("treerow");
    var cell = document.createElement ("treecell");
    cell.setAttribute ("crop", "left");
    cell.setAttribute ("context", "annotationPopup");
    cell.setAttribute ("ondblclick", "gAnnozillaSidebar.loadBodyOnCommand(window.top.gBrowser.selectedBrowser)");
    cell.setAttribute ("class", "text");
    cell.setAttribute ("label", getAnnotationText (annotation, range));

    row.appendChild (cell);
    item.appendChild (row);

    var children2 = document.createElement ("treechildren");
    var item2 = document.createElement ("treeitem");
    item2.setAttributeNS
          (ANNOZILLA_XUL_NS,
           "annozilla-resource",
           annotation.annotationURI);
    item2.setAttributeNS
          (ANNOZILLA_XUL_NS,
           "annozilla-body",
           annotation.bodyURI);
    item2.setAttributeNS
          (ANNOZILLA_XUL_NS,
           "annozilla-in-reply-to",
           inReplyTo);
    var row2  = document.createElement ("treerow");
    var cell2 = document.createElement ("treecell");
    cell2.setAttribute ("crop", "left");
    cell2.setAttribute ("context", "annotationPopup");
    cell2.setAttribute ("ondblclick", "gAnnozillaSidebar.loadBodyOnCommand(window.top.gBrowser.selectedBrowser)");
    cell2.setAttribute ("class", "indent");
    cell2.setAttribute ("label", getAnnotationDescription(annotation));

    row2.appendChild (cell2);
    item2.appendChild (row2);
    children2.appendChild (item2);
    item.appendChild (children2);

    parent.appendChild (item);

    root.setAttribute ("context", "annotationPopup");
    root.setAttribute ("ondblclick", "gAnnozillaSidebar.loadBodyOnCommand(window.top.gBrowser.selectedBrowser)");
}

// Make a date in format yyyy-mm-ddThh:mm:ssZ
// more readable
function getReadableDate (date)
{
    if ( date.match (/^(\d\d\d\d-\d\d-\d\d)T(\d\d:\d\d:\d\d)Z$/))
    {
        return RegExp.$2 + " " + RegExp.$1;
    }
    else
    {
        return date;
    }
};

var annotationTypeLookups = {
    
    getDescription: function (typeURI)
    {
      var rdf =
          Components.classes["@mozilla.org/rdf/rdf-service;1"].
          getService(Components.interfaces.nsIRDFService);
          
      var datasource = browserWindowNamespace.gAnnozilla.annotationTypes;
      
      var source = datasource.GetSource (
            rdf.GetResource ("http://home.netscape.com/NC-rdf#URL"),
            rdf.GetLiteral (typeURI),
            true);

      if (source == undefined) { return source; }

      var target = datasource.GetTarget(
            source,
            rdf.GetResource ("http://home.netscape.com/NC-rdf#Name"),
            true);

      return target.QueryInterface (Components.interfaces.nsIRDFLiteral).Value;
    }
};

// Get the title of the annotation, if available;
// otherwise extract a portion of the annotated text
function getAnnotationText (annotation, range) {
    if (annotation.data.getProperty (Annozilla.DC10_NS, "title"))
    {
        return annotation.data.getProperty (Annozilla.DC10_NS, "title");
    }
    else if (annotation.data.getProperty (Annozilla.DC11_NS, "title"))
    {
        return  annotation.data.getProperty (Annozilla.DC11_NS, "title");
    }
    else if (range) {
        var annotationText = range.toString();
        if (annotationText.length > 50) {
            annotationText = annotationText.substring(0, 49) + "...";
        }
        annotationText = annotationText.replace (/\n/g,' ');
        return annotationText.replace (/^\s+/g,'');
    }
    else {
        var bundle = srGetStrBundle(
            "chrome://annozilla/locale/annozilla.properties"
        );
        return bundle.GetStringFromName ("UNKNOWN_RANGE");
    }
}

// Format the metadata of an annotation into a readable
// string, either the Dublin Core title, or something of the form
// "[type] created by [creator] on [date]"
function getAnnotationDescription (annotation) {
  annotation = annotation.QueryInterface(Components.interfaces.nsIAnnotation);
  
  var bundle = srGetStrBundle("chrome://annozilla/locale/annozilla.properties");
  var type = annotation.data.getSecondaryType();
  var description = "";

  if(type) {
    var description = annotationTypeLookups.getDescription(type);
    if (description == undefined)
    {
        // If we failed to look up the type in the RDF,
        // allow unknown type to be self described (i.e. show whatever it is)
        description = type.substring(type.indexOf("#")+1);
    }
  }

  var creator = annotation.data.getLiteralProperty(Annozilla.DC10_NS, "creator");
  if (!creator) {
      creator = bundle.GetStringFromName("NO_CREATORS");
  }

  var created = annotation.data.getProperty(Annozilla.ANNOT_NS, "created");
  if (created) {
      created = getReadableDate (created);
  } else {
      created = bundle.GetStringFromName("NO_DATE");
  }

  return bundle.formatStringFromName
       ("ANNOTATION_ITEM", [description, creator, created], 3);
}


function addMessage (message, inReplyTo) {

    // The root of the annotation tree
    var root = document.getElementById("annotationTreeChildren");
 
    // The node to which we will attach the annotation
    var parent = root;
    if (inReplyTo != undefined) {
        // look for parent: the second treeitem element in document order
        // which has annozilla-resource equal to inReplyTo
        var first = false;
        var treeitems = root.getElementsByTagName("treeitem");
        for (var ix=0; ix < treeitems.length; ix++) {
            if (treeitems[ix].getAttributeNS (ANNOZILLA_XUL_NS, "annozilla-resource") == inReplyTo) {
                if (first) {
                    parent = treeitems[ix].parentNode;
                    break;
                } else {
                    first = true;
                }
            }
        }
    }

    var item = document.createElement ("treeitem");
    item.setAttribute ("container", "false");
    var row  = document.createElement ("treerow");
    var cell = document.createElement ("treecell");
    cell.setAttribute ("crop", "left");
    cell.setAttribute ("class", "text");
    cell.setAttribute ("label", message);

    row.appendChild (cell);
    item.appendChild (row);

    parent.appendChild (item);

}

function setUpUI (server) {
    document.getElementById ("edit-annotation-body")
        .setAttribute ("disabled", !server.supports (server.EDIT));
    document.getElementById ("delete-annotation")
        .setAttribute ("disabled", !server.supports (server.DELETE));
    /*
    document.getElementById ("reply-to-annotation")
        .setAttribute ("disabled", true);
    */
}

function AnnozillaSidebar(annozilla0) {

    this.bundle = srGetStrBundle("chrome://annozilla/locale/annozilla.properties");

    this.annozilla = annozilla0;

    // add our observer
    this.annozilla.app.addObserver(this);

    //let annozilla resolve and add annotations in order
    annozilla0.sidebar = this;
    annozilla0.addView (this);

    var that = this;

    // add a listener for when the window closes
    window.addEventListener("unload",
    			    function(e) { that.unload(); },
			    false);
}

AnnozillaSidebar.prototype = new AnnozillaAnnotationsView();

AnnozillaSidebar.prototype.bundle= undefined;

AnnozillaSidebar.prototype.clearList = function() {
    clearList();
}

AnnozillaSidebar.prototype.operationInProgress = function(statusmessage) {
    document.getElementById("progress").mode = "undetermined";
    document.getElementById("statusbar-text").
        setAttribute("label", statusmessage);
}

AnnozillaSidebar.prototype.operationComplete = function(statusmessage) {
    document.getElementById("progress").mode = "determined";
    document.getElementById("statusbar-text").
        setAttribute("label", statusmessage);
}

AnnozillaSidebar.prototype.isSelectedTab = function (target) {
    return (target == window.top.gBrowser.selectedBrowser);
}

/**
 * Inform the sidebar of the progress of an operation.
 * @param serversRemaining Number of servers yet to respond
 * (i.e. 0 means operation complete).
 * @param target Target of the operation
 */
AnnozillaSidebar.prototype.updateProgress =
function(serversRemaining, target) {
    // TODO check on target
    if (serversRemaining == 0) {
        this.operationComplete(this.bundle.GetStringFromName("OPERATION_COMPLETE"));
    } else {
        this.operationInProgress (
            this.bundle.formatStringFromName
               ("OPERATION_IN_PROGRESS", [serversRemaining], 1));
    }
}

AnnozillaSidebar.prototype.setHeader = function(message) {
    setHeader(message);
}

AnnozillaSidebar.prototype.annotationOnPopupShowing = function () {

    var tree = document.getElementById('annotationTree');
    var annotationOptionsDisabled = "true";
    var replyOptionsDisabled = "true";
    
    if (tree.currentIndex != -1) {
        annotationOptionsDisabled = "false";
    
        if (tree.contentView.getItemAtIndex(tree.currentIndex).getAttributeNS
          (ANNOZILLA_XUL_NS,
           "annozilla-in-reply-to") == "")
        {
            replyOptionsDisabled = "false";
        }
    }
    document.getElementById ("view-annotation-body").setAttribute ("disabled", annotationOptionsDisabled);
    document.getElementById ("scroll-into-view").setAttribute ("disabled", annotationOptionsDisabled);
    document.getElementById ("edit-annotation-body").setAttribute ("disabled", annotationOptionsDisabled);
    document.getElementById ("delete-annotation").setAttribute ("disabled", annotationOptionsDisabled);
    document.getElementById ("check-for-replies").setAttribute ("disabled", replyOptionsDisabled);
    return true;
}

AnnozillaSidebar.prototype.addAnnotations = function(annotations) {
// Not needed here - we get the annotations added independently
}

AnnozillaSidebar.prototype.addNoAnnotationsMessage = function ()
{
    var bundle = srGetStrBundle("chrome://annozilla/locale/annozilla.properties");
    var noAnnotations = bundle.GetStringFromName ("NO_ANNOTATIONS_LOADED");
    var root = document.getElementById("annotationTreeChildren");
    var item = document.createElement ("treeitem");
    var row  = document.createElement ("treerow");
    row.setAttribute ("id", "noAnnotations");
    var cell = document.createElement ("treecell");
    cell.setAttribute ("label", noAnnotations);
   
    row.appendChild (cell);
    item.appendChild (row);
    root.appendChild (item);
}

AnnozillaSidebar.prototype.noRepliesFor = function (annotation, target)
{
    if (this.isSelectedTab (target))
    {
        var location = findAnnotationParent (annotation.annotationURI, false);

        if (location != undefined)
        {
            var item = document.createElement ("treeitem");
            item.setAttribute ("class", "noreplies");
            var row  = document.createElement ("treerow");
            var cell = document.createElement ("treecell");
            cell.setAttribute ("label", this.bundle.GetStringFromName ("NO_REPLIES"));

            row.appendChild (cell);
            item.appendChild (row);
            location.appendChild (item);
        }
    }
}

AnnozillaSidebar.prototype.getSelectedAnnotationURI = function() {
    var tree = document.getElementById('annotationTree');
    var selectedItem = tree.contentView.getItemAtIndex(tree.currentIndex);
    if (selectedItem && selectedItem.hasAttributeNS (ANNOZILLA_XUL_NS, "annozilla-resource")) {
        return selectedItem.getAttributeNS(ANNOZILLA_XUL_NS, "annozilla-resource");
    } else {
        return null;
    }
}

AnnozillaSidebar.prototype.updateAnnotation = function(annotation) {
    var url = annotation.annotationURI;
    var children = document.getElementById("annotationTreeChildren");
    var treeitems = children.getElementsByTagName("treeitem");
    for (var ix=0 ; ix < treeitems.length; ix++) {
        if (treeitems[ix].getAttributeNS (ANNOZILLA_XUL_NS,"annozilla-resource") == url) {
	    treeItem = treeitems[ix];
	    treeItem.setAttributeNS (ANNOZILLA_XUL_NS,"annozilla-body", annotation.bodyURI);
            
            var cell = treeItem.childNodes.item(0).firstChild;
            // TODO if we resolved the range associated with the annotation,
            // we could have a fallback for when the annotation title doesn't exist
            cell.setAttribute("label", getAnnotationText (annotation, undefined));
	    var cell2 = treeItem.childNodes.item(1).firstChild.firstChild.firstChild;
	    cell2.setAttribute("label", getAnnotationDescription(annotation));
	    break;
	}
    }
}

AnnozillaSidebar.prototype.deleteAnnotationFromUI = function (url) {
    var children = document.getElementById("annotationTreeChildren");
    var treeitems = children.getElementsByTagName("treeitem");
    for (var ix =0 ; ix < treeitems.length; ix++) {
        if (treeitems[ix].getAttributeNS (ANNOZILLA_XUL_NS, "annozilla-resource") == url) {
           try {
               treeitems[ix].parentNode.removeChild (treeitems[ix]);
               if (treeitems.length == 0) {
                   this.addNoAnnotationsMessage();
               }
           } catch (e) {
           }
        }
    }
    // TODO remove the icon
};

AnnozillaSidebar.prototype.deleteAnnotation = function(url, target) {
    // TODO tab check?
    var tree = document.getElementById('annotationTree');
    //need to grab index before delete to select next item
    // NB assumes we have deleted the selected item
    var index = tree.currentIndex;
    this.deleteAnnotationFromUI(url);
    if (index >= tree.view.rowCount) {
        // Deleted the last item, so select the new last item
        tree.view.selection.select(tree.view.rowCount-1); 
    } else {
        tree.view.selection.select(index); 
    }
}

AnnozillaSidebar.prototype.unload = function() {
    dump("Sidebar: unloading\n");
    // remove observer
    this.annozilla.app.removeObserver(this);
    //doesn't work Annozilla.sidebar = null;
    browserWindowNamespace.gAnnozilla.byeByeSidebar();
    browserWindowNamespace.gAnnozilla.removeView (this);
}
    

AnnozillaSidebar.prototype.QueryInterface = function(iid) {  
    if (! (iid.equals(Components.interfaces.nsIAnnotationsObserver) ||
	   iid.equals(Components.interfaces.nsISupports) ) ) {
	       throw Components.results.NS_ERROR_NO_INTERFACE;
	   }
    return this;
}

/**
 * Called when an annotation load is beginning.
 * @param target Tab for which annotations are being loaded
 */
AnnozillaSidebar.prototype.userLoadBeginning = function(target) {
    
    // No action unless the annotations are being loaded for the
    // currently selected tab
    if (this.isSelectedTab (target))
    {
        this.clearList();

        var serverCount = this.annozilla.currentServerCount();
    
        if (serverCount == 0) {
            this.setHeader (this.bundle.GetStringFromName("NO_SERVERS"));
            return;
        }
    }
}

/**
 * Responds to a user pressing the load button
 * by loading annotations for the current page.
 */   
AnnozillaSidebar.prototype.userLoadOnCommand = function() {
    this.annozilla.userLoadOnCommand();
    return true;
}

/**
 * Called when a 'check for replies' operation is beginning.
 * @param uri URI of the annotation for which to check replies.
 * @param target Tab with which the annotation is associated.
 */
AnnozillaSidebar.prototype.replyCheckBeginning = function(uri, target) {
    
    // No action unless the annotations are being loaded for the
    // currently selected tab
    if (this.isSelectedTab (target))
    {
        clearList (uri);
        var serverCount = this.annozilla.currentServerCount();
    
        if (serverCount == 0) {
            this.setHeader (this.bundle.GetStringFromName("NO_SERVERS"));
        }
    }
}

AnnozillaSidebar.prototype.checkForRepliesOnCommand = function(target) {
    var annotationURI = this.getSelectedAnnotationURI();
    this.annozilla.checkForRepliesOnCommand(annotationURI, target);    
    return true;
}

AnnozillaSidebar.prototype.userStopLoadCommand = function () {
    this.annozilla.userStopLoadCommand();
}

/**
 * We might be posting an annotation to a page without loading the
 * annotations first, in which case "No annotations loaded" might be
 * visible. (TODO: In fact, couldn't we have a set of
 * annotations loaded for a different page?)
 */
AnnozillaSidebar.prototype.resetListIfNecessary = function () {
    if (document.getElementById("noAnnotations"))
    {
        var treeroot = document.getElementById("annotationTreeChildren");
        var treechildren = treeroot.childNodes;
        for (var ix = treechildren.length-1; ix >= 0; ix--)
        {
            treeroot.removeChild (treechildren[ix]);
        }
    }
}

/**
 * Called when an existing annotation is going to be updated.
 */
AnnozillaSidebar.prototype.updateBeginning = function(uri, target) {
}

/**
 * Called when a new annotation is going to be posted.
 */
AnnozillaSidebar.prototype.postBeginning = function (target)
{
    this.resetListIfNecessary();
}

AnnozillaSidebar.prototype.postOnCommand = function() {
    this.annozilla.postOnCommand();
}

AnnozillaSidebar.prototype.replyOnCommand = function() {

    var annotationURI = this.getSelectedAnnotationURI();
    this.annozilla.replyOnCommand(annotationURI);
}

AnnozillaSidebar.prototype.updateOnCommand = function(target) {
    var annotationURI = this.getSelectedAnnotationURI();
    this.annozilla.updateOnCommand(annotationURI, target);
}

AnnozillaSidebar.prototype.deleteOnCommand = function(target) {
    var annotationURI = this.getSelectedAnnotationURI();
    this.annozilla.deleteOnCommand(annotationURI, target);
}

AnnozillaSidebar.prototype.loadBodyOnCommand = function(target) {
    var annotationURI = this.getSelectedAnnotationURI();
    if (annotationURI != null)
    {
        this.annozilla.loadBodyOnCommand(annotationURI, target);
    }
}

/**
 * Success callback function to return the results for a loadAnnotations request.
 * This function returns the annotations for one server, designated by serverURI.  
 *
 * @param serverURI string uri of the annotations server responding
 * @param annotatedURI string uri that the annotations annotate
 * @param annotations nsISimpleEnumerator of nsIAnnotations for annotatedURI
 *                    that were on serverURI annotations server
 */
AnnozillaSidebar.prototype.loadAnnotationsOnSuccess =
function(serverURI, annotatedURI, annotationsLength, annotations, target) {   
    return; // annozilla will tell us of the annotations separately
}

/**
 * Callback function to indicate failure for a loadAnnotations request.
 * This function indicates failure for one particular annotations server,
 * designated by serverURI.
 *
 * @param serverURI string uri of the annotations server responding
 * @param annotatedURI string uri for which annotations were requested
 * @param message string error message
 */
AnnozillaSidebar.prototype.loadAnnotationsOnFailure = function(
    serverURI,
    annotatedURI,
    message,
    target
){
    if (this.isSelectedTab (target)) {
        document.getElementById("statusbar-text").
            setAttribute("label", message);
    }  
}

/**
 * Called when an annotation post request succeeds.
 *
 * @param newAnnotation nsIAnnotation the newly created annotation
 */
AnnozillaSidebar.prototype.postAnnotationOnSuccess = function(newAnnotation, target, postWindow) {
    // Could resolve redundantly below and add, 
    //  but let's just let Annozilla call us
}

/**
 * Called when an annotation post fails.
 *
 * @param message string error message
 */
AnnozillaSidebar.prototype.postAnnotationOnFailure = function(message, target, postWindow) {}

/**
 * Called when an annotation update succeeds.
 *
 * @param updatedAnnotation nsIAnnotation that has been updated on server
 */
AnnozillaSidebar.prototype.updateAnnotationOnSuccess =
function(updatedAnnotation, target, updateWindow) {
    
    var uri = updatedAnnotation.annotationURI;
    if (findAnnotationParent(uri, true) != undefined) {
        this.updateAnnotation(updatedAnnotation);
    }
}

/**
 * Called when an annotation update fails.
 *
 * @param url string url of the annotation that could not be updated
 * @param message string error message
 */
AnnozillaSidebar.prototype.updateAnnotationOnFailure = function(url, message, target, updateWindow) {}

/**
 * Called when an annotation is successfully deleted.
 *
 * @param url string url of deleted annotation
 */
AnnozillaSidebar.prototype.deleteAnnotationOnSuccess = function(url, target) {
    this.deleteAnnotation(url, target);
}

/**
 * Called when an annotation delete fails.
 *
 * @param url string url of the annotation that could not be deleted
 * @param message string error message
 */
AnnozillaSidebar.prototype.deleteAnnotationOnFailure = function(url, message, target) {
    // this.annozilla prints an error message.
}

/**
 * Called when the replies to an annotation have been successfully
 * loaded.
 * 
 * @param annotation nsIAnnotation for which replies were requested
 */
AnnozillaSidebar.prototype.getRepliesOnSuccess =
function(annotation, repliesCount, replies, target) {
    
// We allow the Annozilla object to receive and resolve
// the replies. We will be notified of them via addReply.

}

AnnozillaSidebar.prototype.addReply =
function (inReplyTo, reply, range, doc, target)
{

    var model = new AnnozillaAnnotationModel(target);

    var inReplyToURI = reply.data.getResourceProperty
        ("http://www.w3.org/2001/03/thread#", "inReplyTo");

    if (model.hasAnnotation(inReplyToURI))
    {
        // Find the annotation in the tree, and remove the 'No replies' child
        // if it is present
       var annot = findAnnotation (inReplyToURI, true);

       var noreplies = annot.getElementsByClassName ("noreplies");
       for (var ix = 0; ix < noreplies.length; ix++)
       {
           noreplies[ix].parentNode.removeChild (noreplies[ix]);
       }

       addNewAnnotation (reply, range, inReplyToURI);
       
       // Make sure the relevant point in the annotation
       // tree is expanded so that the replies are visible.
       if (annot != undefined)
       {
           annot.setAttribute ("open", "true");
       }
    }
}

/**
 * Called when getting the replies to an annotation fails.
 *
 * @param annotation nsIAnnotation for which replies were requested
 * @param message string error message
 */
AnnozillaSidebar.prototype.getRepliesOnFailure =
function(annotation, message, target) {
    if (this.isSelectedTab (target)) {
        document.getElementById("statusbar-text").
            setAttribute("label", message);
    }
}

/**
 * Called an annotation has successfully been replied to.
 * 
 * @param original nsIAnnotation that existed originally
 * @param reply nsIAnnotation reply created
 */
AnnozillaSidebar.prototype.replyToAnnotationOnSuccess =
function(original, reply, target, replyWindow) {}

/**
 * Called when replying to an annotation fails.
 *
 * @param original nsIAnnotation to which we attempted to reply
 * @param message string error message
 */
AnnozillaSidebar.prototype.replyToAnnotationOnFailure =
function(original, message, target, replyWindow) {}

/**
 * Called after a loadAnnotationBody request
 * when the body text of the annotation has been successfully loaded.
 *
 * @param annotation nsIAnnotation with updated body text
 * @param bodyText string text of the annotation's body
 */
AnnozillaSidebar.prototype.loadAnnotationBodyOnSuccess = function(annotation, bodyText) {}

/**
 * Called after a loadAnnotationBody request when
 * the body text fails to load.
 *
 * @param annotation nsIAnnotation that body was requested for (unmodified)
 * @param message string error message
 */
AnnozillaSidebar.prototype.loadAnnotationBodyOnFailure = function(annotation, message) {}

AnnozillaSidebar._addAnnotationToContainer=Annozilla._addAnnotationToContainer;

AnnozillaSidebar.prototype._debugBadAnnotation = Annozilla._debugBadAnnotation;

AnnozillaSidebar.prototype._processAnnotationsWithAutoLoadInfo =
function(state, annotations)
{
}

AnnozillaSidebar.prototype.addAnnotation = function(annot, range, doc, target) {
    if (this.isSelectedTab (target)) {
        addNewAnnotation(annot, range, null);
    }
}

AnnozillaSidebar.prototype.scrollIntoViewOnCommand = function(target) {
  var annotationURI = this.getSelectedAnnotationURI();
  var model = new AnnozillaAnnotationModel(target);
  if (! model.hasAnnotation(annotationURI) ) {
    // TODO i18n
    window.alert("Unable to scroll to the selected annotation due to an internal error. " +
		 " Try reloading the annotations for the page.");
    return;
  }
  var annotation = model.hasAnnotation(annotationURI).annot;
  var range = gAnnozillaXPointerUtils.xptrResolveAnnotation(annotation, target.contentDocument);
  if (range) {
      var startNode = range.startContainer;
      while (startNode.nodeType!=Node.ELEMENT_NODE) {
          startNode = startNode.parentNode;
      }
     startNode.scrollIntoView();
  }
}

AnnozillaSidebar.prototype.tabSelected = function (browser) {

    clearList();

    var model  = new AnnozillaAnnotationModel(browser);
    var menuitem = window.document.getElementById('stop-load');
    if (model.isResolving()) {
        menuitem.hidden = 0;
    } else {
        menuitem.hidden = 1;
    }
    
    var currentAnnotations = model.getAnnotations();
    var addedAnnotations = {};
    var annotationCount = 0;
    
    if (currentAnnotations != null)
    {
        for (uri in currentAnnotations) {
            // Don't add broken annotations.
            // Don't add replies at this stage.
            if (currentAnnotations[uri].annot.data &&
                currentAnnotations[uri].inReplyTo == undefined) {
                
                annotationCount++;
               
                addNewAnnotation(
                    currentAnnotations[uri].annot,
                    currentAnnotations[uri].range,
                    null
                );
                
                addedAnnotations[uri] = uri;
            }
        }
        
        // Now a second pass dealing with replies

        do
        {
            // Keep a track of whether we've processed any items
            // this time round
            var processedItems = false;

            for (uri in currentAnnotations) {

                // Don't add broken annotations.
                // Don't add replies at this stage.
                if (currentAnnotations[uri].annot.data &&
                    currentAnnotations[uri].inReplyTo != undefined &&
                    // Only if we haven't already added it:
                    addedAnnotations[uri] == undefined &&
                    // Only if the being-replied to annotation already exists:
                    findAnnotationParent (currentAnnotations[uri].inReplyTo, false) != undefined) {
    
                    annotationCount++;
                    
                    addNewAnnotation(
                        currentAnnotations[uri].annot,
                        currentAnnotations[uri].range,
                        currentAnnotations[uri].inReplyTo
                    );
                    
                    addedAnnotations[uri] = uri;
                    processedItems = true;
       
                }
            }
        } while (processedItems);

    }
    if (annotationCount == 0)
    {
        this.addNoAnnotationsMessage();
    }
}

AnnozillaSidebar.prototype.insertIconsPrefChanging = function (insert) {  
};

// See also tabSelected
AnnozillaSidebar.prototype.annotationResolutionBeginning =
function (target) {
    var menuitem = window.document.getElementById('stop-load');
    menuitem.hidden = 0;
};

// See also tabSelected
AnnozillaSidebar.prototype.annotationResolutionFinished =
function (target) {
    var menuitem = window.document.getElementById('stop-load');
    menuitem.hidden = 1;
};
